import logging
from flask_jwt_extended.exceptions import NoAuthorizationError
from flask_restplus import abort
from marshmallow.exceptions import ValidationError
from sqlalchemy.exc import SQLAlchemyError

from portal.rest import rest_api
from portal.service.errors import PortalAPIError

log = logging.getLogger(__name__)


@rest_api.errorhandler(PortalAPIError)
def handle_portal_api_error(ex: PortalAPIError):
    log.error(f"[API] Api error: {ex} ")
    abort(code=ex.code, message=ex.message)


@rest_api.errorhandler(NoAuthorizationError)
def handle_missing_auth_header(ex: NoAuthorizationError):
    log.debug(f"[AUTH] Auth headers are missing: {ex} ")
    return {'message': 'Auth headers is missing', 'error': f'{ex}'}, 401


@rest_api.errorhandler(ValidationError)
def handle_validation_error(ex: ValidationError):
    log.warning(f"[VALID] Validation failed: {ex}")
    return {'message': f"Validation failed on: {ex.messages}"}, 400


@rest_api.errorhandler(SQLAlchemyError)
def handle_db_error(ex: SQLAlchemyError):
    log.error(f"[DB] Error: {ex}", ex)
    return {'message': f'Database error: {ex}'}, 400


@rest_api.errorhandler(NotImplementedError)
def handle_not_implemented_error():
    return {'message': f'Not implemented yet!'}, 404
